#/bin/bash
#
# All Rights Reserved, Copyright (c) FUJITSU LIMITED 2012-2014
#
#	Configuration Tool for Oracle VM Server for SPARC
#
# Revision 2.2  2014/10/02
#
# custom_guest.sh

guest_custom()
{

#---set default------------------------------------------------------------#
CPU_MIN=1
CPU_DEF=1
MEM_MIN=4096
MEM_DEF=4096
MIN_VDISK=1
#--------------------------------------------------------------------------#
TITLE=" 2.Configure a Guest Domain"
DKIND="guest"
PATTERN="Original"
#work_out="${work_dir}/${DOMAIN}"

############################################################################
# Creating Domain
############################################################################

#---get domain name--------------------------------------------------------#

NAME=`ldm ls -p | grep DOMAIN | sed -e 's/name=//' -e 's/ncpu=//' | awk -F"|" '{print $2}'`

#--------------------------------------------------------------------------#

header "$TITLE" "$PATTERN"
print "# Logical domain name and Host ID                                                [1/8]"
print ""
print "Specify a logical domain name for the guest domain."
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

domain_name

work_out="${work_dir}/${DOMAIN}"

host_id

echo ""
echo "=== The following logical domain name and host ID will be set ==="
cat $work_out/dom.tmp
echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# CPU cores
############################################################################

	print_cpu
	header "$TITLE" "$PATTERN"
print "# CPU cores                                                                      [2/8]"
print ""
printf "Assign CPU cores to the domain %s.\n" "$DOMAIN"
print ""
print "### Current CPU allocation for each domain ###"
print ""
cat $work_dir/cpulist.tmp
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

domain_cpu 1

if [ -f $work_out/cpu.info ];then
	echo "=== The following quantity of CPU cores will be allocated ==="
	cat $work_out/cpu.info
else
	echo ""
	echo "CPU core will not be allocated."
fi
echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Setting Memory
############################################################################

header "$TITLE" "$PATTERN"
print_memory

print "# Memory                                                                         [3/8]"
print ""
printf "Assign memory to the domain %s.\n" "$DOMAIN"
print ""
print "### The current amount of memory allocated to each domain ###"
print ""
cat $work_dir/memlist.tmp
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

domain_memory

echo "=== The following memory will be allocated ==="
cat $work_out/mem.tmp
echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Creating VNET
############################################################################

header "$TITLE" "$PATTERN"
print "# Virtual Network                                                                [4/8]"
print ""
print "Do you wish to create a virtual network (vnet)?"
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

while :
do
	YN=""
	echo "Create a virtual network? ([y]/n) ==> \c"
	read YN
	case ${YN:-y} in
		[t])	top
			continue
			;;
		[q])	quit
			continue
			;;
		[n])	NUM_of_VNET=0
			break
			;;
		[y])	while :
			do
				echo "Enter the quantity of virtual networks. [1] ==> \c"
				read NUM_of_VNET

				if [ "${NUM_of_VNET:-1}" = t ];then
					top
					echo ""
					continue
				elif [ "${NUM_of_VNET:-1}" = q ];then
					quit
					echo ""
					continue
				else
					expr ${NUM_of_VNET:-1} + 1 > /dev/null 2>&1
					if [ $? = 0 ];then
						break 2
					else
						echo "ERROR: Invalid input value."
						continue
					fi
				fi
			done
			;;
		  *)	continue
			;;
	esac
done

COUNT01=0
while [ "${YN:-y}" = y -a "$COUNT01" -lt ${NUM_of_VNET:-1} ]
do
	echo ""
	VSW_LIST=`ldm list -o network -p | grep VSW | awk -F"|" '{print $2}' | sed -e s/^name=//`
	COUNT02=`expr $COUNT01 + 1`
	case $COUNT02 in
		  [1])	echo "=== Set the 1st vnet configuration: vnet$COUNT01 ==="
			;;
		  [2])	echo "=== Set the 2nd vnet configuration: vnet$COUNT01 ==="
			;;
		  [3])	echo "=== Set the 3rd vnet configuration: vnet$COUNT01 ==="
			;;
		[4-9])	echo "=== Set the ${COUNT02}th vnet configuration: vnet$COUNT01 ==="
			;;
		    *)	echo "=== Set the next vnet configuration: vnet$COUNT01 ==="
			;;
	esac 

	ldm list -o network -p | grep VSW > $work_dir/vsw_list.tmp

	echo ""
	echo "Please select a virtual switch for vnet$COUNT01 from the following list:"

	k01=0
	for k02 in `echo $VSW_LIST`
	do
		NDEV=`grep -w $k02 $work_dir/vsw_list.tmp | awk -F"|" '{print $5}' | sed -e s/net-dev=//`
		if [ "$NDEV" = "" ];then
			k03="physical NIC was not allocated."
		else
			k03="$NDEV was allocaded."
		fi
		k01=`expr $k01 + 1`
		echo " ${k01}. $k02 : $k03"
	done

	while :
	do
		echo "Enter a number ==> \c"
		read NUM
		case $NUM in
			[t])	top
				continue
				;;
			[q])	quit
				continue
				;;
			  *)	expr $NUM + 1 > /dev/null 2>&1
				if [ $? = 0 ];then
					if [ $NUM -ge 1 -a $NUM -le $k01 ];then
						break
						echo ""
					else
						echo "ERROR: Invalid input value."
						echo ""
					fi
				else
					echo "ERROR: Invalid input value."
					echo ""
				fi
				;;
		esac
	done

	VSW=`echo $VSW_LIST | awk '{print $'$NUM'}'`

	while :
	do
		YN2=""
		echo "Set a VLAN for vnet$COUNT01? (y/[n]) ==> \c"
		read YN2
		case ${YN2:-n} in
		  [t])	top
			continue
			;;
		  [q])	quit
			continue
			;;
		  [y])	while :
			do
				echo "Select a VLAN level for vnet$COUNT01 from the following list:"
				echo " 1. port VLAN"
				echo " 2. tag VLAN"
				echo "Enter a number ==> \c"
				read PoT
				case $PoT in
				  [t])	top
					continue
					;;
				  [q])	quit
					continue
					;;
				  [1])	while :
					do
						echo "Enter a port VLAN ID (e.g., 100) ==> \c"
						read PVID
						expr $PVID + 1 > /dev/null 2>&1
						if [ $? = 0 ];then
							break 2
						else
							echo "ERROR: Invalid input value."
						fi
					done
					;;
				  [2])	while :
					do
						FLAG=OK
						echo "Enter a tag VLAN ID (e.g., 10,20,30...) ==> \c"
						read VID
						A=`echo ${VID:-"blank"} | sed -e s/,/" "/g`
						for i in $A
						do
							expr $i + 1 > /dev/null 2>&1
							if [ $? != 0 ];then
								FLAG=NG
							fi
						done
						
						case $FLAG in
							"OK")	break 2
								;;
							"NG")	echo "ERROR: Invalid input value."
								;;
						esac
					done
					;;
				    *)	echo "ERROR: Invalid input value."
					echo ""
					continue
					;;
				esac
			done

			#ver1.1
			NETDEV=`ldm ls -l -p primary | grep $VSW | awk -F"|" '{print $5}' | sed -e s/net-dev=//`
			if [ "$NETDEV" = "" ];then
				case $PoT in
				  [1])	echo "VNET   : vnet$COUNT01 $VSW PVID $PVID" >> $work_out/vnt.tmp
					echo "VNET   : vnet$COUNT01: virtual switch is $VSW" >> $work_out/vnt.info
					echo "VNET   : vnet$COUNT01: port VLAN ID is $PVID" >> $work_out/vnt.info
					echo "VNET   : vnet$COUNT01: IPMP disabled" >> $work_out/vnt.info
					;;
				  [2])	echo "VNET   : vnet$COUNT01 $VSW VID $VID" >> $work_out/vnt.tmp
					echo "VNET   : vnet$COUNT01: virtual switch is $VSW" >> $work_out/vnt.info
					echo "VNET   : vnet$COUNT01: VLAN ID is $VID" >> $work_out/vnt.info
					echo "VNET   : vnet$COUNT01: IPMP disabled" >> $work_out/vnt.info
					;;
				esac
				break
			else
			#ver1.1

			while :
			do
				YN2=""
				echo "Enable IPMP for vnet$COUNT01? (y/[n]) ==> \c"
				read YN2
				case ${YN2:-n} in
				  [t])	top
					continue
					;;
				  [q])	quit
					continue
					;;
				  [y])	case $PoT in
						[1])	echo "VNET   : vnet$COUNT01 $VSW PVID $PVID linkprop=phys-state" >> $work_out/vnt.tmp
							echo "VNET   : vnet$COUNT01: virtual switch is $VSW" >> $work_out/vnt.info
							echo "VNET   : vnet$COUNT01: port VLAN ID is $PVID" >> $work_out/vnt.info
							echo "VNET   : vnet$COUNT01: IPMP enabled" >> $work_out/vnt.info
							;;
						[2])	echo "VNET   : vnet$COUNT01 $VSW VID $VID linkprop=phys-state" >> $work_out/vnt.tmp
							echo "VNET   : vnet$COUNT01: virtual switch is $VSW" >> $work_out/vnt.info
							echo "VNET   : vnet$COUNT01: VLAN ID is $VID" >> $work_out/vnt.info
							echo "VNET   : vnet$COUNT01: IPMP enabled" >> $work_out/vnt.info
							;;
					esac
					break 2
					;;
				  [n])	case $PoT in
						[1])    echo "VNET   : vnet$COUNT01 $VSW PVID $PVID" >> $work_out/vnt.tmp
							echo "VNET   : vnet$COUNT01: virtual switch is $VSW" >> $work_out/vnt.info
							echo "VNET   : vnet$COUNT01: port VLAN ID is $PVID" >> $work_out/vnt.info
							echo "VNET   : vnet$COUNT01: IPMP disabled" >> $work_out/vnt.info
							;;
						[2])    echo "VNET   : vnet$COUNT01 $VSW VID $VID" >> $work_out/vnt.tmp
							echo "VNET   : vnet$COUNT01: virtual switch is $VSW" >> $work_out/vnt.info
							echo "VNET   : vnet$COUNT01: VLAN ID is $VID" >> $work_out/vnt.info
							echo "VNET   : vnet$COUNT01: IPMP disabled" >> $work_out/vnt.info
							;;	
					esac
					break 2
					;;
				    *) 	continue
					;;
				esac
			done
			fi
			;;
		  [n])
			#ver1.1
			NETDEV=`ldm ls -l -p primary | grep $VSW | awk -F"|" '{print $5}' | sed -e s/net-dev=//`
			if [ "$NETDEV" = "" ];then
				echo "VNET   : vnet$COUNT01 $VSW" >> $work_out/vnt.tmp
				echo "VNET   : vnet$COUNT01: virtual switch is $VSW" >> $work_out/vnt.info
				echo "VNET   : vnet$COUNT01: VLAN disabled" >> $work_out/vnt.info
				echo "VNET   : vnet$COUNT01: IPMP disabled" >> $work_out/vnt.info
				break
			else

		  	while :
			do
				YN2=""
				echo "Enable IPMP for vnet$COUNT01? (y/[n]) ==> \c"
				read YN2
				case ${YN2:-n} in
				  [t])	top
					continue
					;;
				  [q])	quit
					continue
					;;
				  [y])	echo "VNET   : vnet$COUNT01 $VSW linkprop=phys-state" >> $work_out/vnt.tmp
					echo "VNET   : vnet$COUNT01: virtual switch is $VSW" >> $work_out/vnt.info
					echo "VNET   : vnet$COUNT01: VLAN disabled" >> $work_out/vnt.info
					echo "VNET   : vnet$COUNT01: IPMP enabled" >> $work_out/vnt.info
					break 2
					;;
				  [n])	echo "VNET   : vnet$COUNT01 $VSW" >> $work_out/vnt.tmp
					echo "VNET   : vnet$COUNT01: virtual switch is $VSW" >> $work_out/vnt.info
					echo "VNET   : vnet$COUNT01: VLAN disabled" >> $work_out/vnt.info
					echo "VNET   : vnet$COUNT01: IPMP disabled" >> $work_out/vnt.info
					break 2
					;;
				    *)	continue
					;;
				esac
			done
			fi
			;;
		    *)	echo "ERROR: Invalid input value."
			continue
			;;
		esac
	done
	
        COUNT01=`expr $COUNT01 + 1`
done

if [ ${NUM_of_VNET:-1} -eq 0 ];then
	echo ""
	echo "A virtual network will not be created."
else
	if [ -f $work_out/vnt.info ];then
		echo ""
		echo "=== The following virtual network setting will be set ==="
		cat $work_out/vnt.info
	else
		echo ""
		echo "A virtual network will not be created."
	fi
fi

echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Select OS install 
############################################################################

header "$TITLE" "$PATTERN"
print "# OS Install                                                                     [5/8]"
print ""
printf "Select a method to install the Oracle Solaris OS on the logical domain %s.\n" "$DOMAIN"
print "Please select either installation from a network install server for from an iso image"
print "file."
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

os_install 1

if [ -f $work_out/os.info ];then
	echo "=== The following method will be set ==="
	cat $work_out/os.info
else
	echo ""
	echo "Not decided."
fi

echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Creating VDISK
############################################################################

header "$TITLE" "$PATTERN"

print "# Virtual Disk                                                                   [6/8]"
print ""
print "Do you wish to create a virtual disk (vdisk)?"
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

while :
do
	YN=""
	echo "Create a virtual disk? ([y]/n) ==> \c"
	read YN
	case ${YN:-y} in
	  [t])	top
		;;
	  [q])	quit
		;;
	  [y])	make_vdisk 1 1
		break
		;;
	  [n])	break
		;;
	    *)	echo "ERROR: Invalid input value."
		continue
		;;
	esac
done

if [ -f $work_out/gvds.info ];then
	echo ""
	echo "=== The following virtual disk configurations have been set ==="
	cat $work_out/gvds.info
else
	echo ""
	echo "A virtual disk was not created."
	rm $work_out/gvds.tmp > /dev/null 2>&1
	rm $work_out/gvds.info > /dev/null 2>&1
fi
echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Setting VC
############################################################################

header "$TITLE" "$PATTERN"
print "# Virtual Console                                                                [7/8]"
print ""
print "Set a virtual console (vc) to access to each domain's console."
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

VCC_LIST=`ldm ls -l -p | grep VCC | awk -F"|" '{print $2}' | sed -e 's/name=//'`
echo "Please select a virtual console concentrator for $DOMAIN from the following list:"

c01=0
for i in `echo $VCC_LIST`
do
	c01=`expr $c01 + 1`
	echo " $c01. $i"
done 

while :
do
	echo "Enter a number [1] ==> \c"
	read i
	case ${i:-1} in
		[t])	top
			continue
			;;
		[q])	quit
			continue
			;;
		[0])	echo "ERROR: Invalid input value."
			echo ""
			continue
			;;
		  *)	expr ${i:-1} + 1 > /dev/null 2>&1
			if [ $? -ne 0 ];then
				echo "ERROR: Invalid input value."
				echo ""
			elif [ ${i:-1} -gt 0 -a ${i:-1} -le $c01 ];then
				VCC=`echo $VCC_LIST | awk '{print $'${i:-1}'}'`
				break
			else
				echo "ERROR: Invalid input value."
				echo ""
			fi
			;;
	esac
done

add_vcc

echo ""
echo "=== The following virtual console settings will be made ==="
if [ -f $work_out/vc.tmp ];then
	cat $work_out/vc.info
else
	echo "VC for $DOMAIN is not specified."
fi
echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Setting Variables
############################################################################

header "$TITLE" "$PATTERN"
print "# Boot-parameters                                                                [8/8]"
print ""
print "Set the \"auto-boot?\" and \"boot-device\" OBP parameters for the logical domain."
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

set_var

echo ""
echo "=== The following boot parameters will be set ==="
if [ -f $work_out/var.tmp ];then
	cat $work_out/var.info
else
	echo "OBP for $DOMAIN is not specified."
fi
echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Confirmation of setting
############################################################################

header "$TITLE" "$PATTERN"
print "The following I/O domain configuration will be set."
print ""
printf "======= Your configuration information for guest domain %s =======================\n" "$DOMAIN"
if [ -f $work_out/dom.tmp ];then cat $work_out/dom.tmp ;fi
if [ -f $work_out/cpu.info ];then cat $work_out/cpu.info ;fi
if [ -f $work_out/mem.tmp ];then cat $work_out/mem.tmp  ;fi
print "-------------------------------------------------------------------"
if [ -f $work_out/vnt.tmp ];then cat $work_out/vnt.info ;fi
if [ -f $work_out/gvds.tmp ];then cat $work_out/gvds.info ;fi
if [ -f $work_out/vc.tmp ];then cat $work_out/vc.info ;fi
if [ -f $work_out/var.tmp ];then cat $work_out/var.info ;fi
print "-------------------------------------------------------------------"
if [ -f $work_out/os.info ];then cat $work_out/os.info ;fi
print "======================================================================================="
print ""

############################################################################
# Execution and creating of configuration file
############################################################################

EXE=$conf_dir/${DOMAIN}-${DATE}.sh

save_exec 0

############################################################################
# Delete temporary file 
############################################################################

delete
############################################################################

}
