#/bin/bash
#
# All Rights Reserved, Copyright (c) FUJITSU LIMITED 2012-2017
#
#	Configuration Tool for Oracle VM Server for SPARC
#
# Revision 3.0  2017/10/28
#
# custom_primary.sh

primary_custom()
{
DOMAIN=primary

#---set default------------------------------------------------------------#
CPU_MIN=1
CPU_DEF=2
MEM_MIN=4096
MEM_DEF=4096
#--------------------------------------------------------------------------#
TITLE=" 1.Configure a Primary Domain"
DKIND="primary"
PATTERN="Original"
work_out="${work_dir}"

############################################################################
# Removing I/O
############################################################################

status=`print_io`

#--------------------------------------------------------------------------#

header "$TITLE" "$PATTERN"
print "# Select I/O to exclude                                                          [1/9]"
print ""
print "Do you wish to exclude any I/O devices from the primary domain?"
print ""
print "### Allocation status of the current I/O devices ###"
print ""
print_io | tee -a $work_dir/iolist.tmp
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

while :
do
	YN=""
	echo "Remove I/O devices? (y/[n]) ==> \c"
	read YN
	case ${YN:-n} in
		  [t])	top
			;;
		  [q])	quit
			;;
		  [y])	break
			;;
		  [n])	break
			;;
		    *)	continue
			;;
	esac
done	

if [ ${YN:-n} = y ];then
	COUNT1=0
	COUNT2=`cat $work_dir/iolist.tmp |grep BUS | wc -l`
	echo ""

	while [ $COUNT1 -lt $COUNT2 ]
	do
		A=(`cat $work_dir/iolist.tmp | sed -n '/^ '$COUNT1'/p' | awk '{print $2,$4,$5}'`)
		if [ "${A[1]}" = "primary" ];then
			YN=""
			echo "Remove I/O device No. "$COUNT1" ? (y/[n]) ==> \c"
			read YN
			case ${YN:-n} in
				  [t])	top
					continue
					;;
				  [q])	quit
					continue
					;;
				  [y])	if [ "${A[2]}" = "[boot-device]" ];then
						YN=""
						echo "No. "$COUNT1" device is a primary domain boot device. Are you sure ? (y/[n]) ==> \c" 
						read YN
						case ${YN:-n} in
						  [t])	top
							continue
							;;
						  [q])	quit
							continue
							;;
						  [y])	echo "I/O    : ${A[0]} remove from ${A[1]} domain." >> $work_out/remove_io.tmp
							;;
						  [n])	continue
							;;
						    *)	continue
							;;
						esac
					else
						echo "I/O    : ${A[0]} remove from ${A[1]} domain." >> $work_out/remove_io.tmp
					fi
					;;
				  [n])	
					;;
				    *)	continue
					;;
			esac
		else
			echo "The No. "$COUNT1" device has already been removed or allocated to another domain."
			echo "Please press the enter key. \c"
			read
		fi

		COUNT1=`expr $COUNT1 + 1`
		
		if [ -f $work_out/remove_io.tmp ];then
			COUNT3=`cat $work_out/remove_io.tmp | wc -l`
			COUNT4=`cat $work_dir/iolist.tmp | grep primary | wc -l`
			if [ $COUNT3 -eq $COUNT4 ];then
				echo ""
				echo "---------------------------------------------------------------------------------------"
				echo "Cannot remove all I/O devices from the primary domain."
				echo "Please select again."
				echo "---------------------------------------------------------------------------------------"
				echo ""
				COUNT1=0
				rm $work_out/remove_io.tmp
			fi
		fi
	done

	if [ -f $work_out/remove_io.tmp ];then
		echo ""
		echo "=== Confirm removed I/O devices ==="
		cat $work_out/remove_io.tmp
	else
		echo ""
		echo "I/O devices were not removed."
	fi
else
	if [ ${YN:-n} = n ];then
		echo ""
		echo "I/O devices were not removed."
	fi
fi

echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Creating VDS
############################################################################

header "$TITLE" "$PATTERN"
print "# Virtual disk server                                                            [2/9]"
print ""
print "Do you wish to create a virtual disk server (vds) to allow importing virtual disks "
print "into a domain?"
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

while :
do
	YN=""
	echo "Create a virtual disk server (vds)? ([y]/n) ==> \c"
	read YN
	case ${YN:-y} in
		  [t])	top
			;;
		  [q])	quit
			;;
		  [y])	break
			;;
		  [n])	break
			;;
		    *)	continue
			;;
	esac
done

if [ ${YN:-y} = y ];then
	while :
	do
		echo "Enter the desired quantity of virtual disk servers. [1] ==> \c"
		read NUM_of_VDS
		expr ${NUM_of_VDS:-1} + 1 > /dev/null 2>&1
		if [ $? = 0 ];then
			break
		else
			if [ "${NUM_of_VDS:-1}" = t ];then
				top
				continue
			elif [ "${NUM_of_VDS:-1}" = q ];then
				quit
				continue
			else
				echo "ERROR: Invalid input value."
			fi
		fi
	done

	echo ""
	if [ ${NUM_of_VDS:-1} -gt 0 ];then
		echo "=== These vds settings will be created ==="
	fi
	
	COUNT1=0
	while [ $COUNT1 -lt ${NUM_of_VDS:-1} ]
	do
		COUNT2=`expr $COUNT1 + 1`
		case $COUNT2 in
			  [1])	echo "VDS    : Add $DOMAIN-vds$COUNT1 to the primary domain." | tee -a $work_dir/vds.tmp
				;;
			  [2])	echo "VDS    : Add $DOMAIN-vds$COUNT1 to the primary domain." | tee -a $work_dir/vds.tmp
				;;
			  [3])	echo "VDS    : Add $DOMAIN-vds$COUNT1 to the primary domain." | tee -a $work_dir/vds.tmp
				;;
			[4-9])	echo "VDS    : Add $DOMAIN-vds$COUNT1 to the primary domain." | tee -a $work_dir/vds.tmp
				;;
			    *)	echo "VDS    : Add $DOMAIN-vds$COUNT1 to the primary domain." | tee -a $work_dir/vds.tmp
				;;
	        esac
	COUNT1=`expr $COUNT1 + 1`
	done
fi

if [ ${YN:-y} = n -o ${NUM_of_VDS:-1} -eq 0 ];then
	echo ""
	echo "A virtual disk server will not be created."
fi

echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Creating VCC
############################################################################

header "$TITLE" "$PATTERN"
print "# Virtual Console Concentrator                                                   [3/9]"
print ""
print "Create a virtual console concentrator (vcc) service for use by the virtual network"
print "terminal server daemon (vntsd) and as a concentrator for all domain consoles."
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

echo "Enter the range of port numbers for primary-vcc0."

while :
do
	echo " starting port number [5000] ==> \c"
	read STPORT

	case  ${STPORT:-5000} in
		[t])	top
			;;
		[q])	quit
			;;
		  *)	expr ${STPORT:-5000} + 1 > /dev/null 2>&1
			if [ $? = 0 ];then
				if [ ${STPORT:-5000} -ge 1024 -a ${STPORT:-5000} -le 65535 ];then
					break
				else
					echo "ERROR: Invalid input value."
					echo "ERROR: Please enter a number between 1024 and 65535."	
				fi
			else
				echo "ERROR: Invalid input value."
			fi
			;;
	esac
done

while :
do
	DEF_EDPORT=`expr ${STPORT:-5000} + 100`
	if [ $DEF_EDPORT -ge 65535 ];then
		DEF_EDPORT=65535
	fi

	echo " ending port number [$DEF_EDPORT] ==> \c"
	read EDPORT

	case  ${EDPORT:-$DEF_EDPORT} in
		[t])	top
			;;
		[q])	quit
			;;
		  *)	expr ${EDPORT:-$DEF_EDPORT} + 1 > /dev/null 2>&1
			if [ $? = 0 ];then
				if [ ${EDPORT:-$DEF_EDPORT} -ge ${STPORT:-5000} -a ${EDPORT:-$DEF_EDPORT} -le 65535 ];then
					break
				else
					echo "ERROR: Invalid input value."
					echo "ERROR: Please enter a number between ${STPORT:-5000} and 65535."
#					echo "ERROR: Please enter a number bigger than start port number(${STPORT:-5000})."
					echo ""
				fi
			else
				echo "ERROR: Invalid input value."
			fi
			;;
	esac
done

echo ""
echo "=== The following vcc setting will be made ==="
echo "VCC    : $DOMAIN-vcc0 will use the port range of ${STPORT:-5000} to ${EDPORT:-$DEF_EDPORT}" | tee -a $work_dir/vcc.tmp
echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Creating VSW
############################################################################

header "$TITLE" "$PATTERN"
print "# Virtual switch                                                                 [4/9]"
print ""
print "Do you wish to create a virtual switch service (vsw) to enable networking between the"
print "virtual network (vnet) devices in logical domains?"
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

while :
do
	YN=""
	echo "Create a virtual switch service? ([y]/n) ==> \c"
	read YN
	case ${YN:-y} in
		[t])	top
			;;
		[q])	quit
			;;
		[y])	while :
			do
				echo "Enter the quantity of virtual switch services [1] ==> \c"
				read NUM_of_SW
				expr ${NUM_of_SW:-1} + 1 > /dev/null 2>&1
				if [ $? = 0 ];then
					#ver1.1
					if [ ${NUM_of_SW:-1} -le 768 ];then
						break 2
					else
						echo "ERROR: Invalid input value."
					fi
				else
					if [ ${NUM_of_SW:-1} = t ];then
						top
						continue
					elif [ ${NUM_of_SW:-1} = q ];then
						quit
						continue
					else
						echo "ERROR: Invalid input value."
					fi
				fi
			done
			;;
		[n])	break
			;;
		  *)	continue
			;;
	esac
done

COUNT01=0
while [ "${YN:-y}" != n -a $COUNT01 -lt ${NUM_of_SW:-1} ]
do
	echo ""
	COUNT02=`expr $COUNT01 + 1`
	case $COUNT02 in
		  [1])	echo "=== Set the 1st virtual switch service configuration ==="
			;;
		  [2])	echo "=== Set the 2nd virtual switch service configuration ==="
			;;
		  [3])	echo "=== Set the 3rd virtual switch service configuration ==="
			;;
		[4-9])	echo "=== Set the ${COUNT02}th virtual switch service configuration ==="
			;;
		    *)	echo "=== Set the next virtual switch service configuration ==="
			;;
	esac 
	add_vsw $COUNT01
	COUNT01=`expr $COUNT01 + 1`
done

if [ -f $work_out/vsw.tmp ];then
	echo ""
	echo "=== These virtual switch service settings will be created ==="
	cat $work_out/vsw.info
else
	echo ""
	echo "A virtual switch service will not be created."
fi
echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Assign CPU
############################################################################

header "$TITLE" "$PATTERN"
print_cpu
print "# CPU cores                                                                      [5/9]"
print ""
print "Assign CPU cores to the primary domain."
print ""
cat $work_dir/cpulist.tmp
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

domain_cpu 1

if [ -f $work_out/cpu.info ];then
	echo "=== The following quantity of CPU cores will be allocated ==="
	cat $work_out/cpu.info
else
	echo ""
	echo "CPU core will not be allocated."
fi
echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Setting Memory
############################################################################

header "$TITLE" "$PATTERN"
print_memory
print "# Memory                                                                         [6/9]"
print ""
print "Assign memory to the primary domain."
print ""
cat $work_dir/memlist.tmp
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

domain_memory

echo "=== The following memory will be allocated ==="
cat $work_out/mem.tmp
echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Creating FiberChannel VF (SR-IOV)  # V3.0
############################################################################

header "$TITLE" "$PATTERN"
print "# FiberChannel VF (SR-IOV)                                                       [7/9]"
print ""
print "Do you wish to create a VF (SR-IOV) to use virtual FiberChannel devices in I/O domains?"
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

while :
do
	YNf=""
	echo "Create a FibreChannel VF devices? ([y]/n) ==> \c"
	read YNf
	case ${YNf:-y} in
	 	 [t])	top
			;;
		  [q])	quit
			;;
		  [y])	break
			;;
		  [n])	break
			;;
		    *)	continue
			;;
	esac
done

NUM_of_FCPF=0
NUM_of_FCVF=0

if [ ${YNf:-y} = y ];then
	print_fcpf
	if [ ${NUM_of_FCPF} -gt 0 ];then
	create_fcvf
	fi
fi

if [ ${YNf:-y} = n -o ${NUM_of_FCVF} -eq 0 ];then
	echo ""
	echo "Fiber Channel VF device will not be created."
else
	echo ""
	echo "=== These Fibre Channel VF device will be created ==="
	cat $work_out/fcvf.tmp
fi


echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Creating Network VF (SR-IOV) # V3.0
############################################################################

header "$TITLE" "$PATTERN"
print "# Network VF (SR-IOV)                                                            [8/9]"
print ""
print "Do you wish to create a VF (SR-IOV) to use virtual Network devices in I/O domains?"
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

while :
do
	YNn=""
	echo "Create a Network VF devices? ([y]/n) ==> \c"
	read YNn
	case ${YNn:-y} in
	 	 [t])	top
			;;
		  [q])	quit
			;;
		  [y])	break
			;;
		  [n])	break
			;;
		    *)	continue
			;;
	esac
done

NUM_of_NICPF=0
NUM_of_NICVF=0

if [ ${YNn:-y} = y ];then
	print_nicpf

	if [ ${NUM_of_NICPF} -gt 0 ];then
	create_nicvf
	fi
fi

if [ ${YNn:-y} = n -o ${NUM_of_NICVF} -eq 0 ];then
	echo ""
	echo "Network VF device will not be created."
else
	echo ""
	echo "=== These Network VF device will be created ==="
	cat $work_out/nicvf.tmp
fi

echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Setting configuration
############################################################################

#ver1.1
#---get current config-----------------------------------------------------#

print_config()
{
echo "Config name timestamp" | awk '{printf"%-1s%-7s%-44s%-10s\n"," ",$1,$2,$3}'
echo "-------------------------------------------------- ------------" | awk '{printf"%-1s%-51s%-13s\n"," ",$1,$2}'
echo "factory-default                                    -           " | awk '{printf"%-1s%-51s%-13s\n"," ",$1,$2}'

ldm list-config >> $work_dir/list-config.tmp

for i in `ls -t /var/opt/SUNWldm | grep autosave | sed -e s/autosave-//`
do
	LANG=C
	A=`cat $work_dir/list-config.tmp | grep -w $i`
	if [ "$A" != "" ];then
		ls -lt /var/opt/SUNWldm | grep -w "autosave-$i" | sed -e s/autosave-// | awk '{printf"%-1s%-51s%-4s%-3s%-6s\n"," ",C,$6,$7,$8}' C="$A"
	fi
done
}

CUR_CONF=`ls -t /var/opt/SUNWldm | grep autosave | sed -e s/autosave-// | sed -n 1p`

#--------------------------------------------------------------------------#

header "$TITLE" "$PATTERN"
print "# Save configuration                                                             [9/9]"
print ""
print "Save the configuration to a file, and proceed to add a logical domain machine"
print "configuration to the service processor(XSCF)."
print ""
print_config
print ""
print "t:top menu  q:quit"
print "---------------------------------------------------------------------------------------"
print ""

while :
do
	echo "Enter the configuration filename. [initial] ==> \c"
	read CONFIG
	if [ ${CONFIG:-initial} = t ];then
		top
		continue
	elif [ ${CONFIG:-initial} = q ];then
		quit
		continue
	else
		#ver1.1
		grep -w ${CONFIG:-initial} $work_dir/list-config.tmp > /dev/null 2>&1
		if [ $? = 0 ];then
			echo "ERROR: Configuration file named ${CONFIG:-initial} already exists."
			echo ""
			continue
		else
			break
		fi
	fi
done

echo ""
echo "=== The configuration will be saved as the following ==="
echo "The configuration name : ${CONFIG:-initial}" | tee -a $work_out/config.tmp
echo "---------------------------------------------------------------------------------------"
echo "Please press the enter key."
read key

############################################################################
# Confirmation of setting
############################################################################

header "$TITLE" "$PATTERN"
print "The following primary domain configuration will be set."
print ""
print "======= Your configuration information for primary domain ============================="

printf "Domain : %s\n" "$DOMAIN"
cat $work_out/cpu.info
cat $work_out/mem.tmp
print "-------------------------------------------------------------------"
if [ -f $work_out/remove_io.tmp ];then cat $work_out/remove_io.tmp;fi
if [ -f $work_out/vds.tmp ];then cat $work_out/vds.tmp;fi
if [ -f $work_out/vcc.tmp ];then cat $work_out/vcc.tmp;fi
if [ -f $work_out/vsw.info ];then cat $work_out/vsw.info;fi
if [ -f $work_out/fcvf.tmp ];then cat $work_out/fcvf.tmp;fi
if [ -f $work_out/nicvf.tmp ];then cat $work_out/nicvf.tmp;fi
print "-------------------------------------------------------------------"
if [ -f $work_out/config.tmp ];then cat $work_out/config.tmp;fi
print "======================================================================================="
print ""

############################################################################
# Execution and creating of configuration file
############################################################################

EXE=$conf_dir/${DOMAIN}-${DATE}.sh

save_exec 0

############################################################################
# Delete temporary file 
############################################################################

delete
############################################################################

}
